==> firecracker-1.4.1.tar.gz: firecracker-1.4.1/tests/integration_tests/functional/test_max_devices.py <==
# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0
"""Tests scenario for adding the maximum number of devices to a microVM."""

import platform

import pytest

# IRQs are available from 5 to 23, so the maximum number of devices
# supported at the same time is 19.
MAX_DEVICES_ATTACHED = 19


@pytest.mark.skipif(
    platform.machine() != "x86_64", reason="Firecracker supports 24 IRQs on x86_64."
)
def test_attach_maximum_devices(test_microvm_with_api, network_config):
    """
    Test attaching maximum number of devices to the microVM.

    @type: functional
    """
    test_microvm = test_microvm_with_api
    test_microvm.spawn()

    # Set up a basic microVM.
    test_microvm.basic_config()

    # Add (`MAX_DEVICES_ATTACHED` - 1) devices because the rootfs
    # has already been configured in the `basic_config()`function.
    guest_ips = []
    for i in range(MAX_DEVICES_ATTACHED - 1):
        # Create tap before configuring interface.
        _tap, _host_ip, guest_ip = test_microvm.ssh_network_config(
            network_config, str(i)
        )
        guest_ips.append(guest_ip)

    test_microvm.start()

    # Test that network devices attached are operational.
    for i in range(MAX_DEVICES_ATTACHED - 1):
        test_microvm.ssh_config["hostname"] = guest_ips[i]
        # Verify if guest can run commands.
        exit_code, _, _ = test_microvm.ssh.execute_command("sync")
        assert exit_code == 0


@pytest.mark.skipif(
    platform.machine() != "x86_64", reason="Firecracker supports 24 IRQs on x86_64."
)
def test_attach_too_many_devices(test_microvm_with_api, network_config):
    """
    Test attaching to a microVM more devices than available IRQs.

    @type: negative
    """
    test_microvm = test_microvm_with_api
    test_microvm.spawn()

    # Set up a basic microVM.
    test_microvm.basic_config()

    # Add `MAX_DEVICES_ATTACHED` network devices on top of the
    # already configured rootfs.
    for i in range(MAX_DEVICES_ATTACHED):
        # Create tap before configuring interface.
        _tap, _host_ip, _guest_ip = test_microvm.ssh_network_config(
            network_config, str(i)
        )

    # Attempting to start a microVM with more than
    # `MAX_DEVICES_ATTACHED` devices should fail.
    response = test_microvm.actions.put(action_type="InstanceStart")
    assert test_microvm.api_session.is_status_bad_request(response.status_code)
    error_str = (
        "Failed to allocate requested resource: The requested resource"
        " is not available."
    )
    assert error_str in response.text

==> firecracker-1.12.1.tar.gz: firecracker-1.12.1/tests/integration_tests/functional/test_max_devices.py <==
# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0
"""Tests scenario for adding the maximum number of devices to a microVM."""

import platform

import pytest

# IRQs are available from 5 to 23. We always use one IRQ for VMGenID device, so
# the maximum number of devices supported at the same time is 18.
MAX_DEVICES_ATTACHED = 18


@pytest.mark.skipif(
    platform.machine() != "x86_64", reason="Firecracker supports 24 IRQs on x86_64."
)
def test_attach_maximum_devices(uvm_plain_any):
    """
    Test attaching maximum number of devices to the microVM.
    """
    test_microvm = uvm_plain_any
    test_microvm.spawn()

    # Set up a basic microVM.
    test_microvm.basic_config()

    # Add (`MAX_DEVICES_ATTACHED` - 1) devices because the rootfs
    # has already been configured in the `basic_config()`function.
    for _ in range(MAX_DEVICES_ATTACHED - 1):
        test_microvm.add_net_iface()
    test_microvm.start()

    # Test that network devices attached are operational.
    for i in range(MAX_DEVICES_ATTACHED - 1):
        # Verify if guest can run commands.
        test_microvm.ssh_iface(i).check_output("sync")


@pytest.mark.skipif(
    platform.machine() != "x86_64", reason="Firecracker supports 24 IRQs on x86_64."
)
def test_attach_too_many_devices(uvm_plain):
    """
    Test attaching to a microVM more devices than available IRQs.
    """
    test_microvm = uvm_plain
    test_microvm.spawn()

    # Set up a basic microVM.
    test_microvm.basic_config()

    # Add `MAX_DEVICES_ATTACHED` network devices on top of the
    # already configured rootfs.
    for _ in range(MAX_DEVICES_ATTACHED):
        test_microvm.add_net_iface()

    # Attempting to start a microVM with more than
    # `MAX_DEVICES_ATTACHED` devices should fail.
    error_str = (
        "Failed to allocate requested resource: The requested resource"
        " is not available."
    )
    with pytest.raises(RuntimeError, match=error_str):
        test_microvm.start()
